import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { scaleLinear, scaleLog, scaleTime, min, max, extent, timeFormat, bin, sum, timeMonths } from 'd3';
import { useData } from './useData'
import { AxisBottom } from './AxisBottom'
import { AxisLeft } from './AxisLeft'
import { Marks } from './Marks'

const csvUrl = 'https://gist.githubusercontent.com/curran/a9656d711a8ad31d812b8f9963ac441c/raw/c22144062566de911ba32509613c84af2a99e8e2/MissingMigrants-Global-2019-10-08T09-47-14-subset.csv';
const width = 960;
const height = 500;
const margin = { top: 20, right: 20, bottom: 100, left: 100 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 65;

const App = () => {
  const { data } = useData(csvUrl);

  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
      // d['Reported Date'] = new Date(d['Reported Date']);
      // d['Total Dead and Missing'] = +d['Total Dead and Missing'];
      // d['Location Coordinates'] = d['Location Coordinates'];
  const xValue = d => d['Reported Date'];
  const yValue = d => d['Total Dead and Missing'];
  
  const xAxisLabel = 'Time';
  const yAxisLabel = 'Total Dead and Missing';
  
  
  
  const xAxisTickFormat = timeFormat('%m/%d/%Y');
  
  const tickOffset = 10;  
  
  const circleRadius = 2;
  
  const xScale = scaleTime()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth])
  	.nice();
  
  const [start, stop] = xScale.domain();
  
  const binnedData = bin()
  	.value(xValue)
  	.domain(xScale.domain())
    .thresholds(timeMonths(start, stop))(data);
  
  const customBinnedData = binnedData.map(array => ({
    x0: array.x0,
    x1: array.x1,
  	y:  sum(array, d => d['Total Dead and Missing'])
  }));
  
  const yScale = scaleLog()
  	.domain([1, max(data, yValue)])
  	.range([innerHeight, 0])
  	.nice();
  
  const filteredData = data.filter(d => yValue(d) > 1);
  
  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisLeft xScale={yScale} yScale={yScale} width={innerWidth} tickOffset={tickOffset}/>
        <text 
          className="axis-label" 
          textAnchor="middle" 
          transform={`translate(${-yAxisLabelOffset},${innerHeight/2}) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
        <AxisBottom xScale={xScale} height={innerHeight} tickFormat={xAxisTickFormat} tickOffset={tickOffset}/>
        <text 
          className="axis-label" 
          x={innerWidth /2} 
          y={innerHeight + xAxisLabelOffset} 
          textAnchor="middle" 
        >
          {xAxisLabel}
        </text>
				<Marks 
          data={filteredData} 
          xScale={xScale} yScale={yScale} 
          xValue={xValue} yValue={yValue} 
          height={innerHeight} 
          tooltipFormat={d => d}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
